package libchord;

import java.io.Serializable;
import java.math.BigInteger;

import libchord.utils.Utils;

public class Identifier implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8400744906746707927L;
	private final String IP;
	private final int port;
	private final BigInteger identifier;

	public static int keySize() {
		return Utils.getHashSize();
	}
	public Identifier(final String IP, final int port) {
		this.IP = IP;
		this.port = port;
		this.identifier = Utils.sha1Hash(IP + ":" + port);
	}

	public Identifier(final String IP) {
		this.IP = IP;
		this.port = 0;
		this.identifier = Utils.sha1Hash(IP + ":" + port);
	}
	
	public Identifier(final BigInteger hashValue) {
		this.IP = hashValue.toString();
		this.port = -1;
		this.identifier = hashValue;
	}

	public boolean between(final Identifier left, final Identifier right) {
		final BigInteger a = left.toBigInt();
		final BigInteger b = right.toBigInt();
		if (a.compareTo(b) < 0) {
			return (identifier.compareTo(a) > 0 && identifier.compareTo(b) < 0);
		} else {
			return (identifier.compareTo(a) > 0 || identifier.compareTo(b) < 0);
		}
	}

	public boolean betweenOrRight(final Identifier left, final Identifier right) {
		final BigInteger a = left.toBigInt();
		final BigInteger b = right.toBigInt();
		if (a.compareTo(b) < 0) {
			return (identifier.compareTo(a) > 0 && identifier.compareTo(b) <= 0);
		} else {
			return (identifier.compareTo(a) > 0 || identifier.compareTo(b) <= 0);
		}
	}

	@Override
	public boolean equals(final Object obj) {
		return toString().equals(obj.toString());
	}

	public String getIP() {
		return IP;
	}

	public int getPort() {
		return port;
	}

	public BigInteger toBigInt() {
		return identifier;
	}

	public String toString() {
		return IP + ":" + port;
	}

}
