package libchord;

import java.math.BigInteger;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.HashMap;
import java.util.Set;

import libchord.rmi.RemoteStorageNode;

public class StorageNode extends Node implements RemoteStorageNode {

	private static final long serialVersionUID = -881797479297053915L;

	private HashMap<BigInteger, String> storageSpace;
	private String dataIf;

	public StorageNode(Identifier identifier) throws RemoteException {
		super(identifier);
		storageSpace = new HashMap<BigInteger, String>();
	}

	public StorageNode(Identifier identifier, Identifier seed) throws RemoteException {
		super(identifier, seed);
		storageSpace = new HashMap<BigInteger, String>();
	}

	@Override
	public String getIfExists(Identifier dataID) throws RemoteException {
		Identifier destination = findSuccessor(dataID.toBigInt());
		if (destination.equals(this.identifier)) {
			synchronized (storageSpace) {
				return storageSpace.get(dataID.toBigInt());
			}
		} else {
			try {
				return ((RemoteStorageNode) lookup(destination)).getIfExists(dataID);
			} catch (Exception e) {
				return null;
			}
		}
	}

	@Override
	public String take(final Identifier dataID) throws RemoteException {
		synchronized (storageSpace) {
			dataIf = takeIfExists(dataID);
		}
		if (dataIf != null) {
			return dataIf;
		} else {
			while (dataIf == null) {
				try {
					Thread.sleep(500);
					synchronized (storageSpace) {
						dataIf = takeIfExists(dataID);
					}
					Thread.yield();
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			return dataIf;
		}
	}

	@Override
	public String get(final Identifier dataID) throws RemoteException {
		synchronized (storageSpace) {
			dataIf = getIfExists(dataID);
		}
		if (dataIf != null) {
			return dataIf;
		} else {
			while (dataIf == null) {
				try {
					Thread.sleep(500);
					synchronized (storageSpace) {
						dataIf = getIfExists(dataID);
					}
					Thread.yield();
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			return dataIf;
		}
	}

	@Override
	public String takeIfExists(Identifier dataID) throws RemoteException {
		Identifier destination = findSuccessor(dataID.toBigInt());
		if (destination.equals(this.identifier)) {
			synchronized (storageSpace) {
				return storageSpace.remove(dataID.toBigInt());
			}
		} else {
			try {
				return ((RemoteStorageNode) lookup(destination)).takeIfExists(dataID);
			} catch (Exception e) {
				return null;
			}
		}
	}

	@Override
	public Identifier put(String data) throws RemoteException {
		Identifier dataID = new Identifier(data, 0);
		System.err.println("Data ID " + dataID.toBigInt());
		Identifier destination = findSuccessor(dataID.toBigInt());
		System.err.println("Destination: " + destination);
		if (destination.equals(this.identifier)) {
			System.err.println("Local Storage");
			synchronized (storageSpace) {
				storageSpace.put(dataID.toBigInt(), data);
			}
			return dataID;
		} else {
			try {
				System.err.println("Remote store @ " + destination);
				return ((RemoteStorageNode) lookup(destination)).put(data);
			} catch (Exception e) {
				return null;
			}
		}
	}

	@Override
	protected int fixFingers(int next) {
		int nextt = super.fixFingers(next);
		if (nextt == 1) {
			fixStorage(true);
		}
		return nextt;
	}

	public void leave() throws InterruptedException, RemoteException, NotBoundException {
		die = true;
		bgThread.stop();
		System.err.println("BEFORE");
		Registry registry = LocateRegistry.getRegistry("localhost", 1099);
		registry.unbind("rmi://" + identifier.toString() + "/libchord.Node");
		System.err.println("AFTER");
		Thread.sleep(5000);
		synchronized (storageSpace) {
			fixStorage(false);
		}
	}

	protected void fixStorage(boolean allowMyself) {
		synchronized (storageSpace) {
			Set<BigInteger> keys = storageSpace.keySet();
			for (BigInteger key : keys) {
				String data = storageSpace.get(key);
				Identifier dataID = new Identifier(data, 0);
				try {
					Identifier destination = findSuccessor(dataID.toBigInt());
					if (!destination.equals(this.identifier)) {
						System.err.println("Moving " + dataID.toBigInt() + " to node " + destination);
						storageSpace.remove(key);
						((RemoteStorageNode) lookup(destination)).put(data);
					} else if (!allowMyself) {
						((RemoteStorageNode) (firstAvailableSuccessor(false))).put(data);
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}

}
